/*
 * File:   GPS406a.h
 * Author: Daniel Hankewycz
 * Description: These functions are written for the Penn State Buss Tracking
 *              project. These functions provides methods to parse and interpret
 *              an incomming string from a EM-406A GPS module.
 *
 * Created on May 8, 2013, 1:06 AM
 */
#ifndef BUSTRACKER_H
#define	BUSTRACKER_H

#define GPGGA 1
#define GPGSA 2
#define GPGSV 3
#define GPRMC 4
#define GPVTG 5
#define GPGLL 6

#include "myPic.h"

//----------------------------------------------
// Function Prototypes
//----------------------------------------------

//-----------------------------------------------------------------------------
// GETMODE
// Given: a properly tokenized string containing GPS data
// Return: the mode of data
//-----------------------------------------------------------------------------
int16 getMode(char line[]);


//-----------------------------------------------------------------------------
// PARSEDATARMC
// Given: pointer to a properly formated RMC char array, time array,
//			lat array, lng array
// Result: Returns the time, latitude, longitude, velocity, and heading data
//          from the RMC string into their respective arrays
//-----------------------------------------------------------------------------
void parseDataRMC(char line[]);

//-----------------------------------------------------------------------------
// CONVERTCOORDINATE
// Given: inCoordinate a pointer to coordinate "ddmm.mmmmm"
// Result: converts coordinate from "ddmm.ssss" format to "dd.dddddd"
//-----------------------------------------------------------------------------
void convertCoordinate(char coordinate[]);

//-----------------------------------------------------------------------------
// CHECKSUM
// Given: {in} a pointer to a null terminated NMEA string
// Returns: TRUE is checksum is valid, FALSE if not
//-----------------------------------------------------------------------------
int8 checkSum(char in[]);


//-----------------------------------------------------------------------------
// PUSHERROR
// Given: code ant 16 bit integer representing a specific error
// Result: code is pushed into the error queue
//-----------------------------------------------------------------------------
void pushError(int16 code);


//-----------------------------------------------------------------------------
// FRONTERROR
// Returns: value of item in front of errorQueue (index zero)
//-----------------------------------------------------------------------------
int16 frontError(void);


//-----------------------------------------------------------------------------
// BACKERROR
// Returns: value of item in the back of errorQueue (index (errorSize-1))
//-----------------------------------------------------------------------------
int16 backError(void);


//-----------------------------------------------------------------------------
// POPERROR
// Result: decrements errorQueueSize, removes front element and shifts all
//          elements up
//-----------------------------------------------------------------------------
void popError(void);


//-----------------------------------------------------------------------------
// STRRMV
// Given:   s a pointer to a string
//          ini the initial index of the sub-string to be removed
//          fin the final index of the sub-string to be removed
// Result: Removes all character in {s} between {ini} and {fin}, and appends
//          a null terminator at the end of the string
//          example:    GIVEN   [s = "Daniel H.", ini = 3, fin = 5]
//                      RESULT  [s = "Dan H."]
//-----------------------------------------------------------------------------
void strrmv(char *s, int8 ini, int8 fin);


//-----------------------------------------------------------------------------
// STRLENGTH
// Given: a null terminated char array s
// Returns: number of characters preceding null character
//-----------------------------------------------------------------------------
int8 strLength(char s[]);
int8 mystrlenpgm(far rom char *s);
void mymemcpypgm2ram(char *dest, far rom char *src, int8 length);

#endif	/* BUSTRACKER_H */

