/*
*******************************************************************************
* Copyright(C) 2011 Renesas Electronics Corporation
* RENESAS ELECTRONICS CONFIDENTIAL AND PROPRIETARY
* This program must be used solely for the purpose for which
* it was furnished by Renesas Electronics Corporation. No part of this
* program may be reproduced or disclosed to others, in any
* form, without the prior written permission of Renesas Electronics
* Corporation. 
*
* This device driver was created by Applilet3 for RL78/G13
* 16-Bit Single-Chip Microcontrollers
* Filename:	CG_serial_user.c
* Abstract:	This file implements device driver for Serial module.
* APIlib:	Applilet3 for RL78/G13 E1.00b [14 Jan 2011]
* Device:	R5F100LE
* Compiler:	IAR Systems iccrl78
* Creation date:	9/21/2011
*******************************************************************************
*/

/*
*******************************************************************************
** Include files
*******************************************************************************
*/
#include <includes.h>
#include <CG_macrodriver.h>
#include <CG_serial.h>
#include <CG_userdefine.h>
#include <monitor.h>

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
extern volatile UCHAR	*gpCsi10RxAddress;	/* csi10 receive buffer address */
extern volatile USHORT	gCsi10RxLen;		/* csi10 receive data length */
extern volatile USHORT	gCsi10RxCnt;		/* csi10 receive data count */
extern volatile UCHAR	*gpCsi10TxAddress;	/* csi10 send buffer address */
extern volatile USHORT	gCsi10TxLen;		/* csi10 send data length */
extern volatile USHORT	gCsi10TxCnt;		/* csi10 send data count */
extern volatile UCHAR	gIica0MasterStatusFlag;	/* iica0 master flag */ 
extern volatile UCHAR	gIica0SlaveStatusFlag;	/* iica0 slave flag */
extern volatile UCHAR	*gpIica0RxAddress;	/* iica0 receive buffer address */
extern USHORT	gIica0RxCnt;		/* iica0 receive data length */
extern USHORT	gIica0RxLen;		/* iica0 receive data count */
extern volatile UCHAR	*gpIica0TxAddress;	/* iica0 send buffer address */
extern USHORT	gIica0TxCnt;		/* iica0 send data count */
extern volatile UCHAR * gp_uart2_tx_address;         /* uart2 send buffer address */
extern USHORT  g_uart2_tx_count;            /* uart2 send data number */
extern volatile UCHAR * gp_uart2_rx_address;         /* uart2 receive buffer address */
extern USHORT  g_uart2_rx_count;            /* uart2 receive data number */
extern USHORT  g_uart2_rx_length;           /* uart2 receive data length */

extern volatile UCHAR rxInProgress;
extern volatile UCHAR txInProgress;

/* Start user code for global. Do not edit comment generated here */
volatile UCHAR G_UART_SendingData = 0;
volatile UCHAR G_UART_ReceivingData = 0;
volatile UCHAR G_SPI_SendingData = 0;
volatile UCHAR G_SPI_ReceivingData = 0;
volatile UCHAR G_IIC_SendingData = 0;
volatile UCHAR G_IIC_ReceivingData = 0;



/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is INTCSI10 interrupt service routine.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
#pragma vector = INTCSI10_vect
__interrupt void MD_INTCSI10(void)
{
	
        delay_ms(1);
        SIR02 = SSR02;
        UCHAR dummy;
        
	dummy = (UCHAR)(SSR02 & _0001_SAU_OVERRUN_ERROR);
        SIR02 = (USHORT)dummy;
	if (dummy == 1U)
	{
		CSI10_ErrorCallback(dummy);	/* overrun error occurs */
	}
        //	else                                    /* it seems we're getting overruns once in a while.. please fix :) */
	{
		if (gCsi10TxCnt > 0U)
		{
                        if (gCsi10RxLen > 0)          /* Check if we need to receive within this transmit*/
                        {             
			  *gpCsi10RxAddress = SIO10;
                          gCsi10RxCnt++;
    			  gpCsi10RxAddress++;
                        }
                        else
                        {
                           dummy = SIO10;             /* do we need to trigger a read if we don't need the data? */
                        }
			SIO10 = *gpCsi10TxAddress;
			gpCsi10TxAddress++;
			gCsi10TxCnt--;		       
		}
                else if (gCsi10RxLen > 1)              /* Read without sending data. Check on > 1 because fall through triggers another receive */
                {
                      *gpCsi10RxAddress = SIO10;
                      gCsi10RxCnt++;
                      gpCsi10RxAddress++;
                      gCsi10RxLen--;
                      SIO10 = 0xFF;                     /* dummy byte */
                }
		else 
		{
			if (gCsi10TxCnt == 0U && (gCsi10RxLen > 0) )
			{
				*gpCsi10RxAddress = SIO10;
			}
			CSI10_SendEndCallback();	/* complete send */
			CSI10_ReceiveEndCallback();	/* complete receive */
		}
	}
}
/***********************************************************************************************************************
* Function Name: r_uart2_interrupt_receive. Dedicated for NMEA data
* Description  : This function is INTSR2 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/

#pragma vector = INTSR2_vect
__interrupt static void r_uart2_interrupt_receive(void)
{
#if 1   
    
    if ( (g_uart2_rx_length > 0) &&   // search for start of command
         (g_uart2_rx_count == 0)  && 
         (RXD2 != '$')) 
    {
       return;  
    }       
    
    if (g_uart2_rx_length > g_uart2_rx_count)
    {
        *gp_uart2_rx_address = RXD2;
        gp_uart2_rx_address++;
        g_uart2_rx_count++;

        /* check on requested length or end of NMEA command */
        if ((g_uart2_rx_length == g_uart2_rx_count) || 
            (*(gp_uart2_rx_address - 1) == 0x0A) && (*(gp_uart2_rx_address - 2) == 0x0D))
        {
            r_uart2_callback_receiveend();
        }
    }
    
#else
    
    if (g_uart2_rx_length > g_uart2_rx_count)
    {
        *gp_uart2_rx_address = RXD2;
        gp_uart2_rx_address++;
        g_uart2_rx_count++;

        if (g_uart2_rx_length == g_uart2_rx_count)
        {
            r_uart2_callback_receiveend();
        }
    }
#endif
    
}


/***********************************************************************************************************************
* Function Name: r_uart2_interrupt_error
* Description  : This function is INTSRE2 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/

#pragma vector = INTSRE2_vect
__interrupt static void r_uart2_interrupt_error(void)
{
    UCHAR err_type;

    *gp_uart2_rx_address = RXD2;

    err_type = SSR11;
    SIR11 = SSR11;

    r_uart2_callback_error(err_type);
}

/***********************************************************************************************************************
* Function Name: r_uart2_interrupt_send
* Description  : This function is INTST2 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
#pragma vector = INTST2_vect
__interrupt static void r_uart2_interrupt_send(void)
{
    if (g_uart2_tx_count > 0U)
    {
        TXD2 = *gp_uart2_tx_address;
        gp_uart2_tx_address++;
        g_uart2_tx_count--;
    }
    else
    {
        r_uart2_callback_sendend();
    }
}
/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is a callback function when CSI10 finishes reception.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void CSI10_ReceiveEndCallback(void)
{
	
    G_SPI_ReceivingData = 0;
	
}
/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is a callback function when CSI10 reception error occurs.
**
**  Parameters:
**	err_type: error type value
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void CSI10_ErrorCallback(UCHAR err_type)
{
	
	
}
/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is a callback function when CSI10 finishes transmission.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void CSI10_SendEndCallback(void)
{
	
    G_SPI_SendingData = 0;
	
}
/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is INTIICA0 interrupt service routine.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
#pragma vector = INTIICA0_vect
__interrupt void MD_INTIICA0(void)
{
	if ((IICS0 & _80_IICA_STATUS_MASTER) == 0x80U)
	{
		IICA0_MasterHandler();
	}
}
/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is IICA0 master handler.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void IICA0_MasterHandler(void)
{
	/* Control for communication */
	if ((IICBSY0 == 0U) && (gIica0TxCnt != 0U))
	{
		IICA0_MasterErrorCallback(MD_SPT);
	}
	/* Control for sended address */
	else
	{
		if ((gIica0MasterStatusFlag & _80_IICA_ADDRESS_COMPLETE) == 0U)
		{
			if (ACKD0 == 1U)
			{
				gIica0MasterStatusFlag |= _80_IICA_ADDRESS_COMPLETE;
				if (TRC0 == 1U)
				{
					WTIM0 = 1U;
					if (gIica0TxCnt > 0U)
					{
						IICA0 = *gpIica0TxAddress;
						gpIica0TxAddress++;
						gIica0TxCnt--;
					}
					else
					{
						IICA0_MasterSendEndCallback();
					}
				}
				else
				{
					ACKE0 = 1U;
					WTIM0 = 0U;
					WREL0 = 1U;
				}
			}
			else
			{
				IICA0_MasterErrorCallback(MD_NACK);
			}
		}
		else
		{
			/* Master send control */
			if (TRC0 == 1U)
			{
				if ((ACKD0 == 0U) && (gIica0TxCnt != 0U))
				{
					IICA0_MasterErrorCallback(MD_NACK);
				}
				else
				{
					if (gIica0TxCnt > 0U)
					{
						IICA0 = *gpIica0TxAddress;
						gpIica0TxAddress++;
						gIica0TxCnt--;
					}
					else
					{
						IICA0_MasterSendEndCallback();
					}
				}
			}
			/* Master receive control */
			else
			{
				if (gIica0RxCnt < gIica0RxLen)
				{
					*gpIica0RxAddress = IICA0;
					gpIica0RxAddress++;
					gIica0RxCnt++;
					if (gIica0RxCnt == gIica0RxLen)
					{
						ACKE0 = 0U;
						WREL0 = 1U;
						WTIM0 = 1U;
					}
					else
					{
						WREL0 = 1U;
					}
				}
				else
				{
					IICA0_MasterReceiveEndCallback();
				}
			}
		}
	}
}
/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is a callback function when IICA0 master error occurs.
**
**  Parameters:
**	flag: status flag
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void IICA0_MasterErrorCallback(MD_STATUS flag)
{
	
	
}
/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is a callback function when IICA0 finishes master reception.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void IICA0_MasterReceiveEndCallback(void)
{
	SPT0 = 1U;
	
    G_IIC_ReceivingData = 0;
	
}
/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is a callback function when IICA0 finishes master transmission.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void IICA0_MasterSendEndCallback(void)
{
	SPT0 = 1U;
	
    G_IIC_SendingData = 0;
	
}
/***********************************************************************************************************************
* Function Name: r_uart2_callback_receiveend
* Description  : This function is a callback function when UART2 finishes reception.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_uart2_callback_receiveend(void)
{
    
  OS_ERR err;
  OSQPost((OS_Q      *) &MonitorQueue,
                (void      *) 1,
                (OS_MSG_SIZE) sizeof(void *),   // not used by controller
                (OS_OPT     ) OS_OPT_POST_FIFO,
                (OS_ERR    *) &err);
  
  rxInProgress = 0;
    
}

/***********************************************************************************************************************
* Function Name: r_uart2_callback_sendend
* Description  : This function is a callback function when UART2 finishes transmission.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_uart2_callback_sendend(void)
{
    
  txInProgress = 0;
    
}

/***********************************************************************************************************************
* Function Name: r_uart2_callback_error
* Description  : This function is a callback function when UART2 reception error occurs.
* Arguments    : err_type -
*                    error type value
* Return Value : None
***********************************************************************************************************************/
// debug

UCHAR lastError = 0;


static void r_uart2_callback_error(UCHAR err_type)
{
    
    lastError = err_type;
    
}



