/*
*******************************************************************************
* Copyright(C) 2010, 2011 Renesas Electronics Corporation
* RENESAS ELECTRONICS CONFIDENTIAL AND PROPRIETARY
* This program must be used solely for the purpose for which
* it was furnished by Renesas Electronics Corporation. No part of this
* program may be reproduced or disclosed to others, in any
* form, without the prior written permission of Renesas Electronics
* Corporation.
*
* This device driver was created by CodeGenerator for RL78/G13
* 16-Bit Single-Chip Microcontrollers
* Filename:         r_timer_user.c
* Abstract:         This file implements device driver for TAU module.
* APIlib:           CodeGenerator for RL78/G13 V1.01.00 [31 May 2011]
* Device:           R5F100LLAFAB
* Compiler:         ICCRL78
* Creation date:    06/07/2011
*******************************************************************************
*/

/*
*******************************************************************************
** Pragma directive
*******************************************************************************
*/
/* Start user code for pragma. Do not edit comment generated here */

/*
*******************************************************************************
** Include files
*******************************************************************************
*/
#include <includes.h>
#include <CG_macrodriver.h>
#include <CG_timer.h>
#include <CG_userdefine.h>


/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
bool pwmEnabled = false;
unsigned short pwmCounter;

/* Declare a variable for storing the event count */
#define DUTY_PERCENTAGE 20

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**  This function is INTTM05 interrupt service routine.
**
**  Parameters:
**  None
**
**  Returns:
**  None
**
**-----------------------------------------------------------------------------
*/

__interrupt void INTTM04_Interrupt(void)
{
    /* Clear timer TM05 interrupt flag */
  
    uint32_t reg = 0U;

    /* Copy the current duty value stored in TDR04 */
    reg = TDR04;

    /* Calculate the new percentage */
    reg = (reg + 1) * DUTY_PERCENTAGE / 100U;

    /* Load the new duty */
    TDR05 = (uint16_t)reg;
  
    TMIF04 = 0;
    
}

__interrupt void INTTM05_Interrupt(void)
{
    /* Clear timer TM05 interrupt flag */
    TMIF05 = 0;
    
    pwmCounter++;
    if (pwmCounter > 250) // magic number!
    {
        TMMK04 = 1U;    /* disable INTTM00 interrupt */
        TMMK05 = 1U;    /* disable INTTM01 interrupt */
     
        P0_bit.no5 = 0;
        TDR05 = 0;
    
        TT0 |= _0010_TAU_CH4_STOP_TRG_ON | _0020_TAU_CH5_STOP_TRG_ON;
        TOE0 &= ~_0020_TAU_CH5_OUTPUT_ENABLE | ~_0010_TAU_CH4_OUTPUT_ENABLE;
        TO0 &= ~_0020_TAU_CH5_OUTPUT_VALUE_1;
     
        /* Mask channel 4 interrupt */ 
        TMIF04 = 0U;    /* clear INTTM00 interrupt flag */
        /* Mask channel 5 interrupt */    
        TMIF05 = 0U;    /* clear INTTM01 interrupt flag */
        P0_bit.no5 = 0;
        pwmEnabled = false;
    }
}


__interrupt void INTTM07_Interrupt(void)
{
    /* Clear timer TM07 interrupt flag */
    TMIF07 = 0;
    
    /* Stop PWM on TO5 */
    TT0 |= _0010_TAU_CH4_STOP_TRG_ON | _0020_TAU_CH5_STOP_TRG_ON;
    TOE0 &= ~_0020_TAU_CH5_OUTPUT_ENABLE;
    
    pwmEnabled = false;
    
    /* Stop T07 */
    TT0 |= _0080_TAU_CH7_STOP_TRG_ON;        

    /* Mask channel 4 interrupt */
    TMMK04 = 1U;    /* disable INTTM00 interrupt */
    TMIF04 = 0U;    /* clear INTTM00 interrupt flag */
    /* Mask channel 5 interrupt */
    TMMK05 = 1U;    /* disable INTTM01 interrupt */
    TMIF05 = 0U;    /* clear INTTM01 interrupt flag */
    /* Mask channel 7 interrupt */
    TMMK07 = 1U;    /* disable INTTM07 interrupt */
    TMIF07 = 0U;    /* clear INTTM07 interrupt flag */
    
}


