/*
*********************************************************************************************************
*
*                                GPS NMEA parser support code for nmeap01.c
*                                     
*                                           Renesas RL78
*                                               on the
*                                     YRDKRL78G13 Evaluation Board
*
* Filename      : gps.c
* Version       : V1.00
*********************************************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <nmeap.h>

static nmeap_context_t          nmea;	           /* parser context */
nmeap_gga_t                     gga;		   /* this is where the data from GGA messages will show up */
nmeap_rmc_t                     rmc;		   /* this is where the data from RMC messages will show up */
static volatile int             user_data;         /* user can pass in anything. typically it will be a pointer to some user data */


/** called when a gpgga message is received and parsed. Not used at the moment */
static void gpgga_callout(nmeap_context_t *context,void *data,void *user_data)
{
   // nmeap_gga_t *gga = (nmeap_gga_t *)data;
}


/** called when a gprmc message is received and parsed. Not used at the moment */
static void gprmc_callout(nmeap_context_t *context,void *data,void *user_data)
{
    // nmeap_rmc_t *rmc = (nmeap_rmc_t *)data;
}

/*
*********************************************************************************************************
*                                          NMEA_Init
*
* Description : Initializes the NMEA parser
*
* Arguments   : none
*
* Returns     : -1 if error. 0 if init was ok
*
* Notes       : none
*********************************************************************************************************
*/
int NMEA_Init(void)
{
    char             status;
    
    /* initialize the nmea context    */                                                
    nmeap_init(&nmea,(void *)&user_data);

    /* add standard GPGGA parser      */                                                
    status = nmeap_addParser(&nmea,"GPGGA",nmeap_gpgga,gpgga_callout,&gga);   

    /*add standard GPRMC parser      */                                                
    status = nmeap_addParser(&nmea,"GPRMC",nmeap_gprmc,gprmc_callout,&rmc);

    return status;
}


/*
*********************************************************************************************************
*                                          pushNmeaBuff
*
* Description : Pushes buffer with NMEA data through the parser
*
* Arguments   : none
*
* Returns     : none
*
* Notes       : none
*********************************************************************************************************
*/
void pushNmeaBuff(char *buffer,unsigned char length)
{
    unsigned char len = length;
    nmeap_parseBuffer(&nmea,buffer,&len); 
}

/*
*********************************************************************************************************
*                                          pushNmeaChar
*
* Description : Pushes a single character of NMEA data through the parser
*
* Arguments   : none
*
* Returns     : none
*
* Notes       : none
*********************************************************************************************************
*/  
void pushNmeaChar(char ch)
{
        int             status;
        
        /*  pass it to the parser                  */
        /* status indicates whether a complete msg */
        /* arrived for this byte                   */
        /* NOTE : in addition to the return status */
        /* the message callout will be fired when  */
        /* a complete message is processed         */       
        status = nmeap_parse(&nmea,ch);
        

        /* process the return code        */                                                
        switch(status) 
        {
            case NMEAP_GPGGA:
                /* GOT A GPGGA MESSAGE */
                user_data = 1;
                break;
            case NMEAP_GPRMC:
                /* GOT A GPRMC MESSAGE */
                user_data = 2;
                break;
            default:
                break;
        }
}


