// controller.h
#ifndef _CONTROLLER_H
#define _CONTROLLER_H

/* Stepper motor defines */
#define STEPPER_POSITION(x)  P7 = (P7 & 0xE1) | (x << 1);   /* Stepper motor is connected at P71, P72, P73, P74 */
#define STEPPER_HOLD_TIME    20u  /* hold time in msec       */
#define STEPPER_REST          0  /* rest  position     0000 */
#define STEP1                 3  /* first position is  0011 */
#define STEP2                 9  /* second position is 0110 */
#define STEP3                 12 /* third position is  1100 */
#define STEP4                 6  /* fourth position is 1001 */

/* Control message commands. All cmd's that indicate a SD write will be appended with YYYY-MM-DD HH:MM:SS at the beginning of the line */
#define CMD_ROTATE                0         /* rotates platform 1 unit clockwise */
#define CMD_MOVE_UP               1         /* tilts platform 1 unit up */
#define CMD_MOVE_DOWN             2         /* tilts platform 1 unit down */
#define CMD_STORE_GPS_LOCATION    3         /* stores GPS info on SD card in format             Lon:%10f Lat:%10f Sats:%2i\n */
#define CMD_REMOVE_CARD           4         /* indicates that user want's to remove sd card. Files should be closed. Files unmounted */
#define CMD_INSERT_CARD           5         /* indicates that user has inserted a sd card. FS to be mounted and log files can be opened*/
#define CMD_STORE_LOG_MESSAGE     6         /* indicates that a log message should be stored on sd card */

extern OS_Q             ControlQueue;

typedef struct 
{
  unsigned char Cmd;                        /* cmd type. To be set by sender */
  unsigned char DataLength;                 /* length of data. should match strlen(data) */
  unsigned char *Data;                      /* null terminated string to be written to SD card */
  bool          MessageHandled;             /* set to true if message has been processed by receiver */
}ControlMessage;



#endif // _CONTROLLER_H

