;***************************************;***************************************
;                                                                       
;    Filename:         					RFID.asm                                       
;    Date:                              1/15/2014                                  
;    File Version:                      1.0                            
;    Author:                            jeff bachiochi                             
;    Company:                           imagine that                       
;                                       www.imaginethatnow.com                         
;***************************************;***************************************
;                                                              
;    Files required:                                        
;                                                   
;***************************************;***************************************
;
;    Notes:
;
;***************************************;***************************************
;
;    Revision History: 
;
;		Ver0.1	-	initial reader
;		Ver0.2	-	working reader
;		Ver1.2	-	initial release
;
;***************************************;***************************************
;
; PROCESSOR DECLARATION
;
;***************************************;***************************************
;
     LIST      P=PIC18F25K22          ; list directive to define processor
     #INCLUDE <P18F25K22.INC>         ; processor specific variable definitions
;
;***************************************;***************************************
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;***************************************;***************************************
;
;   Oscillator Selection bits:
;     FOSC = LP            LP oscillator
;     FOSC = XT            XT oscillator
;     FOSC = HSHP          HS oscillator (high power > 16 MHz)
;     FOSC = HSMP          HS oscillator (medium power 4-16 MHz)
;     FOSC = ECHP          EC oscillator, CLKOUT function on OSC2 (high power, >16 MHz)
;     FOSC = ECHPIO6       EC oscillator (high power, >16 MHz)
;     FOSC = RC            External RC oscillator, CLKOUT function on OSC2
;     FOSC = RCIO6         External RC oscillator
     CONFIG FOSC = INTIO67				; Internal oscillator block
;     FOSC = INTIO7        Internal oscillator block, CLKOUT function on OSC2
;     FOSC = ECMP          EC oscillator, CLKOUT function on OSC2 (medium power, 500 kHz-16 MHz)
;     FOSC = ECMPIO6       EC oscillator (medium power, 500 kHz-16 MHz)
;     FOSC = ECLP          EC oscillator, CLKOUT function on OSC2 (low power, <500 kHz)
;     FOSC = ECLPIO6       EC oscillator (low power, <500 kHz)
;
;   4X PLL Enable:
;     PLLCFG = OFF         Oscillator used directly
     CONFIG PLLCFG = ON          		; Oscillator multiplied by 4
;
;   Primary clock enable bit:
;     PRICLKEN = OFF       Primary clock can be disabled by software
;     PRICLKEN = ON        Primary clock enabled
;
;   Fail-Safe Clock Monitor Enable bit:
;     FCMEN = OFF          Fail-Safe Clock Monitor disabled
;     FCMEN = ON           Fail-Safe Clock Monitor enabled
;
;   Internal/External Oscillator Switchover bit:
;     IESO = OFF           Oscillator Switchover mode disabled
;     IESO = ON            Oscillator Switchover mode enabled
;
;   Power-up Timer Enable bit:
;     PWRTEN = ON          Power up timer enabled
;     PWRTEN = OFF         Power up timer disabled
;
;   Brown-out Reset Enable bits:
;     BOREN = OFF          Brown-out Reset disabled in hardware and software
;     BOREN = ON           Brown-out Reset enabled and controlled by software (SBOREN is enabled)
;     BOREN = NOSLP        Brown-out Reset enabled in hardware only and disabled in Sleep mode (SBOREN is disabled)
;     BOREN = SBORDIS      Brown-out Reset enabled in hardware only (SBOREN is disabled)
;
;   Brown Out Reset Voltage bits:
     CONFIG BORV = 285           		; VBOR set to 2.85 V nominal
;     BORV = 250           VBOR set to 2.50 V nominal
;     BORV = 220           VBOR set to 2.20 V nominal
;     BORV = 190           VBOR set to 1.90 V nominal
;
;   Watchdog Timer Enable bits:
     CONFIG WDTEN = OFF          		; Watch dog timer is always disabled. SWDTEN has no effect.
;     WDTEN = NOSLP        WDT is disabled in sleep, otherwise enabled. SWDTEN bit has no effect
;     WDTEN = SWON         WDT is controlled by SWDTEN bit of the WDTCON register
;     WDTEN = ON           WDT is always enabled. SWDTEN bit has no effect
;
;   Watchdog Timer Postscale Select bits:
;     WDTPS = 1            1:1
;     WDTPS = 2            1:2
;     WDTPS = 4            1:4
;     WDTPS = 8            1:8
;     WDTPS = 16           1:16
;     WDTPS = 32           1:32
;     WDTPS = 64           1:64
;     WDTPS = 128          1:128
;     WDTPS = 256          1:256
;     WDTPS = 512          1:512
;     WDTPS = 1024         1:1024
;     WDTPS = 2048         1:2048
;     WDTPS = 4096         1:4096
;     WDTPS = 8192         1:8192
;     WDTPS = 16384        1:16384
;     WDTPS = 32768        1:32768
;
;   CCP2 MUX bit:
     CONFIG CCP2MX = PORTB3      		; CCP2 input/output is multiplexed with RB3
;     CCP2MX = PORTC1      CCP2 input/output is multiplexed with RC1
;
;   PORTB A/D Enable bit:
;     PBADEN = OFF         PORTB<5:0> pins are configured as digital I/O on Reset
;     PBADEN = ON          PORTB<5:0> pins are configured as analog input channels on Reset
;
;   P3A/CCP3 Mux bit:
;     CCP3MX = PORTC6      P3A/CCP3 input/output is mulitplexed with RC6
;     CCP3MX = PORTB5      P3A/CCP3 input/output is multiplexed with RB5
;
;   HFINTOSC Fast Start-up:
;     HFOFST = OFF         HFINTOSC output and ready status are delayed by the oscillator stable status
;     HFOFST = ON          HFINTOSC output and ready status are not delayed by the oscillator stable status
;
;   Timer3 Clock input mux bit:
;     T3CMX = PORTB5       T3CKI is on RB5
;     T3CMX = PORTC0       T3CKI is on RC0
;
;   ECCP2 B output mux bit:
;     P2BMX = PORTC0       P2B is on RC0
;     P2BMX = PORTB5       P2B is on RB5
;
;   MCLR Pin Enable bit:
;     MCLRE = INTMCLR      RE3 input pin enabled; MCLR disabled
;     MCLRE = EXTMCLR      MCLR pin enabled, RE3 input pin disabled
;
;   Stack Full/Underflow Reset Enable bit:
;     STVREN = OFF         Stack full/underflow will not cause Reset
;     STVREN = ON          Stack full/underflow will cause Reset
;
;   Single-Supply ICSP Enable bit:
     CONFIG LVP = OFF            		; Single-Supply ICSP disabled
;     LVP = ON             Single-Supply ICSP enabled if MCLRE is also 1
;
;   Extended Instruction Set Enable bit:
;     XINST = OFF          Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
;     XINST = ON           Instruction set extension and Indexed Addressing mode enabled
;
;   Background Debug:
     CONFIG DEBUG = ON           		; Enabled
;     DEBUG = OFF          Disabled
;
;   Code Protection Block 0:
;     CP0 = ON             Block 0 (000800-001FFFh) code-protected
;     CP0 = OFF            Block 0 (000800-001FFFh) not code-protected
;
;   Code Protection Block 1:
;     CP1 = ON             Block 1 (002000-003FFFh) code-protected
;     CP1 = OFF            Block 1 (002000-003FFFh) not code-protected
;
;   Code Protection Block 2:
;     CP2 = ON             Block 2 (004000-005FFFh) code-protected
;     CP2 = OFF            Block 2 (004000-005FFFh) not code-protected
;
;   Code Protection Block 3:
;     CP3 = ON             Block 3 (006000-007FFFh) code-protected
;     CP3 = OFF            Block 3 (006000-007FFFh) not code-protected
;
;   Boot Block Code Protection bit:
;     CPB = ON             Boot block (000000-0007FFh) code-protected
;     CPB = OFF            Boot block (000000-0007FFh) not code-protected
;
;   Data EEPROM Code Protection bit:
;     CPD = ON             Data EEPROM code-protected
;     CPD = OFF            Data EEPROM not code-protected
;
;   Write Protection Block 0:
;     WRT0 = ON            Block 0 (000800-001FFFh) write-protected
;     WRT0 = OFF           Block 0 (000800-001FFFh) not write-protected
;
;   Write Protection Block 1:
;     WRT1 = ON            Block 1 (002000-003FFFh) write-protected
;     WRT1 = OFF           Block 1 (002000-003FFFh) not write-protected
;
;   Write Protection Block 2:
;     WRT2 = ON            Block 2 (004000-005FFFh) write-protected
;     WRT2 = OFF           Block 2 (004000-005FFFh) not write-protected
;
;   Write Protection Block 3:
;     WRT3 = ON            Block 3 (006000-007FFFh) write-protected
;     WRT3 = OFF           Block 3 (006000-007FFFh) not write-protected
;
;   Configuration Register Write Protection bit:
;     WRTC = ON            Configuration registers (300000-3000FFh) write-protected
;     WRTC = OFF           Configuration registers (300000-3000FFh) not write-protected
;
;   Boot Block Write Protection bit:
;     WRTB = ON            Boot Block (000000-0007FFh) write-protected
;     WRTB = OFF           Boot Block (000000-0007FFh) not write-protected
;
;   Data EEPROM Write Protection bit:
;     WRTD = ON            Data EEPROM write-protected
;     WRTD = OFF           Data EEPROM not write-protected
;
;   Table Read Protection Block 0:
;     EBTR0 = ON           Block 0 (000800-001FFFh) protected from table reads executed in other blocks
;     EBTR0 = OFF          Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
;
;   Table Read Protection Block 1:
;     EBTR1 = ON           Block 1 (002000-003FFFh) protected from table reads executed in other blocks
;     EBTR1 = OFF          Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
;
;   Table Read Protection Block 2:
;     EBTR2 = ON           Block 2 (004000-005FFFh) protected from table reads executed in other blocks
;     EBTR2 = OFF          Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
;
;   Table Read Protection Block 3:
;     EBTR3 = ON           Block 3 (006000-007FFFh) protected from table reads executed in other blocks
;     EBTR3 = OFF          Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
;
;   Boot Block Table Read Protection bit:
;     EBTRB = ON           Boot Block (000000-0007FFh) protected from table reads executed in other blocks
;     EBTRB = OFF          Boot Block (000000-0007FFh) not protected from table reads executed in other blocks    
;
;***************************************;***************************************
;
; Constant DEFINITIONS
;
;***************************************;***************************************
;
;	PORTA
DOOR		equ		0					; door unlock control
;			equ		1					; 
;			equ		2					; 
;			equ		3					; 
LED1		equ		4					; LED 1 (good code read)
CS			equ		5					; SPI CS
LED3		equ		6					; LED 3 (searching)
LED2		equ		7					; LED 2 (bad code read)
;
;	PORTB
SLP			equ		0					; Sleep Output
RDY			equ		1					; Ready Input
DEMOD		equ		2					; Demodulation Input
MOD			equ		3					; Modulation Output
CFG1		equ		4					; Configuration 1 Input (add codes)
CFG2		equ		5					; Configuration 2 Input	(remove codes)
;CFG3		equ		6					; 
;CFG4		equ		7					; 
;
;	PORTC
LED4		equ		0					; LED 4 (door unlocked)
LED5		equ		1					; LED 5 (add codes)
LED6		equ		2					; LED 6 (remove codes)
SCK			equ		3					; SPI clock
SDI			equ		4					; SPI Data In
SDO			equ		5					; SPI Data Out
TX			equ		6					; UART Transmitter
RX			equ		7					; UART Receiver
;
;	Flags
RC1_In		equ		0					; 0=RC1 buffer empty, 1=NOT empty
ODD			equ		1					; first symbol=0 (even), second symbol=1 (odd)
OUT			equ		2					; controls the state of the Door Lock
GoodCode	equ		3					; last code was Good
BadCode		equ		4					; last code was Bad
NoCode		equ		5					; last search found no code
Tick		equ		6					; timer 0
Timing		equ		7					; Code Wait
;
TXBuffer_Size	equ	0x80
RCBuffer_Size	equ	0x80
RFIDBuffer_Size	equ	0x100
RFIDDataBuffer_Size	equ	0x40
UnlockCount	equ		D'10'				; number of half seconds
CR			equ		0x0D
LF			equ		0x0A
;
;***************************************;***************************************
;
; VARIABLE DEFINITIONS
;
; Refer to datasheet for available data memory (RAM) organization
;
;***************************************;***************************************
;
    CBLOCK 0x00 						; (ACCESS)
Temp1									; temporary variable
Temp2									; temporary variable	
ITemp									; temporary interrupt variable
Flags									; operational flags
RCBuffer_Head							; RCBuffer head pointer
RCBuffer_Tail							; RCBuffer tail pointer
RC1CharOut								; character from receive buffer
TXBuffer_Head							; TXBuffer head pointer
TXBuffer_Tail							; TXBuffer tail pointer
TX1CharIn								; character to tranmit buffer
ByteCount								; temporary variable								
Count									; temporary variable
UnlockCounter							; unlock timer counter
Value									; SPI value
Mode									; state
HeaderCount								; Number of header bytes
IDCount									; Number of ID bytes
RowParity								; row parity check
ColumnParity							; column parity check
CodeOffset								; ID offset
EECHAR									; character to/from EEPROM
EvenData								; value of EVEN byte
;
W_TEMP            						; w register for context saving
STATUS_TEMP    					  		; status used for context saving 
BSR_TEMP      							; bank select used for ISR context saving

    ENDC
    CBLOCK 0x100 
TXBuffer:TXBuffer_Size					; characters to be transmitted go here
RCBuffer:RCBuffer_Size   				; characters received go here
    ENDC
    CBLOCK 0x200 
RFIDBuffer:RFIDBuffer_Size   			; manchester symbols go here
    ENDC
    CBLOCK 0x300 
RFIDDataBuffer:RFIDDataBuffer_Size 		; symbol data go here
RFIDIDBuffer:0x05  						; RFID goes here
    ENDC    
;
;***************************************;***************************************
;
; EEPROM INITIALIZATION
;
; The 18F25K22 has 256 bytes of non-volatile EEPROM starting at 0xF00000
;
;***************************************;***************************************
;
DATAEE 	org    0xF00000 ; Starting address for EEPROM for 18F4553
;
    	DE 		0x38, 0x00, 0xAF, 0xCD, 0x52, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF	
;
;***************************************;***************************************
;
; RESET VECTOR
;
;***************************************;***************************************
;
RES_VECT
		org    0x0000            		; processor reset vector
		nop
        goto    Init          			; go to beginning of initialization
;
;***************************************;***************************************
;
; HIGH PRIORITY INTERRUPT VECTOR
;
;***************************************;***************************************
;
ISRHV   org    0x0008
        goto    HIGH_ISR             
;
;***************************************;***************************************
;
; LOW PRIORITY INTERRUPT VECTOR
;
;***************************************;***************************************
;
ISRLV   org    0x0018          
	    goto    LOW_ISR             
;
;***************************************;***************************************
;
; HIGH PRIORITY INTERRUPT SERVICE ROUTINE
;
;***************************************;***************************************
;
HIGH_ISR  
;
;***************************************
;
HIGH_Chk4TX1
		btfss	PIE1,		TX1IE		; skip next if TX enabled
		goto	HIGH_Chk4TX1_Exit
;
		btfsc	PIR1,		TX1IF		; skip next if TX busy
		call	TX1_INT

HIGH_Chk4TX1_Exit
;
;***************************************
;		
HIGH_Chk4RC1
		btfss	PIE1,		RC1IE		; skip next if RC enabled
		goto	HIGH_Chk4RC1_Exit
;
		btfsc	PIR1,		RC1IF		; skip next if RC empty
		call	RC1_INT

HIGH_Chk4RC1_Exit
;
;***************************************
;		
HIGH_Chk4SSP2
		btfss	PIE3,		SSP2IE		; skip next if SPI enabled
		goto	HIGH_Chk4SSP2_Exit
;
		btfsc	PIR3,		SSP2IF		; skip next if no SPI
		call	SSP2_INT

HIGH_Chk4SSP2_Exit
;
;***************************************
;		
HIGH_Chk4TMR0
		btfss	INTCON,		TMR0IE		; skip next if TMR0 enabled
		goto	HIGH_Chk4TMR0_Exit
;
		btfsc	INTCON,		TMR0IF		; skip next if no overflow
		call	TMR0_INT

HIGH_Chk4TMR0_Exit
;
;***************************************
;
        retfie  FAST
;
;***************************************;***************************************
;
; LOW PRIORITY INTERRUPT SERVICE ROUTINE
;
;***************************************;***************************************
;
LOW_ISR
          ; Context Saving for Low ISR
        movwf   W_TEMP              ; save W register
        movff   STATUS, STATUS_TEMP ; save status register
        movff   BSR, BSR_TEMP       ; save bankselect register

; none at this time

          ; Context Saving for Low ISR
        movff   BSR_TEMP, BSR       ; restore bankselect register
        movf    W_TEMP, W           ; restore W register
        movff   STATUS_TEMP, STATUS ; restore status register
        retfie
;
;***************************************;***************************************
;
; 	TX1 Interrupt
;
;	Prerequisites
;		TXBuffer		-	buffer for received characters
;		TXBuffer_Size	-	size of the RCBuffer
;	Changes
;		FSR2			-	indirect pointer 2 used for TXBuffer buffer
;		TXREG1			- 	transmit register
;		TXBuffer_Tail	- 	TXBuffer pointer (out)
;		PIE1.TX1IE		-	disable/enable transmit interrupts
;
;***************************************;***************************************
;
TX1_INT
		movf	TXBuffer_Head,	W		; get head pointer
		cpfseq	TXBuffer_Tail			; skip next if same (empty)
		goto	TX1_INT_CharToSend		; else got some characters to send, go on
;
TX1_INT_BufferEmpty
		bcf		PIE1,	TX1IE			; stop further interrupts
		goto	TX1_INT_Exit			; done here
;		
TX1_INT_CharToSend
		lfsr	2,	TXBuffer			; use indirect pointer 2 for TXBuffer
		movff	TXBuffer_Tail,	FSR2L	; get tail pointer
		movff	POSTINC2,	TXREG1		; charcter into Transmit register
		movlw	low TXBuffer			; start of TCBuffer
		addlw	TXBuffer_Size			; add in the TXBuffer size
		cpfseq	FSR2L					; skip next if pointer overflow
		goto	TX1_INT_PointerOK		; else no rollover, go on
;
TX1_INT_PointerOverflow
		movlw	low TXBuffer			; start of TCBuffer
		movwf	FSR2L					; save it
;
TX1_INT_PointerOK
		movff	FSR2L,	TXBuffer_Tail	; save new tail pointer
;		
TX1_INT_Exit		
		return
;
;***************************************;***************************************
;
; 	RC1 Interrupt
;
;	Prerequisites
;		RCBuffer		-	buffer for received characters
;		RCBuffer_Size	-	size of the RCBuffer
;	Changes
;		RCSTA1.FERR		- 	none/ferr flag bit (framing error)
;		RCSTA1.OERR		- 	none/oerr flag bit (overrun error)
;		FSR2			-	indirect pointer 2 used for RCBuffer buffer
;		ITemp			-	temporary interrupt varaiable
;		RCBuffer_Head	- 	RCBuffer pointer (in)
;		RCSTA1.SPEN		-	dsable/enable SPI port
;
;***************************************;***************************************
;
RC1_INT
		btfsc	RCSTA1,	FERR			; skip next if no Framing Error
		goto	RC1_INT_FERR			; else go error
;
		btfsc	RCSTA1,	OERR			; skip next if no Overrun Error
		goto	RC1_INT_OERR			; else go error
;
		lfsr	2,		RCBuffer		; use indirect pointer 2 for RCBuffer buffer
		incf	RCBuffer_Head,	W		; increment head pointer
		movwf	ITemp					; save it
		movlw	low RCBuffer			; get start of RCBuffer
		addlw	RCBuffer_Size			; add the RCBbuffer size
		cpfseq	ITemp					; skip next if pointer overflows
		goto	RC1_INT_PointerOK		; else it doesn't, go on
;
RC1_INT_PointerOverflow
		movlw	low RCBuffer			; get start of RCBuffer
		movwf	ITemp					; savve it
;
RC1_INT_PointerOK				
		cpfseq	RCBuffer_Tail			; skip next if no more room
		goto	RC1_INT_RCBufferRoom	; else we got room, go on
;
RC1_INT_RCBufferFull		
		goto	RC1_INT_Exit			; done here
;
RC1_INT_RCBufferRoom		
		movff	RCBuffer_Head,	FSR2L	; get original head pointer
		movff	RCREG1,	INDF2			; save Char into RCBuffer
		movff	ITemp,	RCBuffer_Head	; save new pointer
		goto	RC1_INT_Exit			; done here
;
RC1_INT_FERR							; fall through
;
RC1_INT_OERR
		bcf		RCSTA1,	SPEN			; reset UART
		bsf		RCSTA1,	SPEN			; reenable UART
;
RC1_INT_Exit					
		return
;
;***************************************;***************************************
;
; 	SPI2 Interrupt
;
;	Changes
;		Value			- 	value from the SPI receiver
;		Count			-	number of 1's
;		ByteCount		-	four byte (32 bits per symbol)
;		FSR0			-	RFIDBuffer
;		PIR3.SSP2IF		-	clear the interrupt
;
;***************************************;***************************************
;
SSP2_INT
		movff	SSP2BUF,	Value		; retrieve SPI byte
		btfsc	Value,		7			; skip next if bit 7=0
		incf	Count					; else add 1 to Count
		btfsc	Value,		6			; skip next if bit 6=0
		incf	Count					; else add 1 to Count		
		btfsc	Value,		5			; skip next if bit 5=0
		incf	Count					; else add 1 to Count		
		btfsc	Value,		4			; skip next if bit 4=0
		incf	Count					; else add 1 to Count		
		btfsc	Value,		3			; skip next if bit 3=0
		incf	Count					; else add 1 to Count		
		btfsc	Value,		2			; skip next if bit 2=0
		incf	Count					; else add 1 to Count
		btfsc	Value,		1			; skip next if bit 1=0
		incf	Count					; else add 1 to Count
		btfsc	Value,		0			; skip next if bit 0=0
		incf	Count					; else add 1 to Count
		clrf	SSP2BUF					; dummy write
;
		incf	ByteCount,	F			; increment ByteCount
		movlw	4						;
		cpfseq	ByteCount				; skip next if ByteCount=4
		goto	SSP2_INT_Exit			; else not 4, done here
;
		movlw	0x10					; 16 (1/2 32 bits)
		cpfslt	Count					; skip next if Count<16
		goto	SSP2_INT_1				; else its >=16, symbol=1
;
SSP2_INT_0								; symbol=0
		clrf	Count					; Count=0x00
		goto	SSP2_INT_Continue		; go on
;
SSP2_INT_1								; symbol=1
		setf	Count					; Count=0xFF			
		goto	SSP2_INT_Continue		; go on
;
SSP2_INT_Continue
		movff	Count,	POSTINC0		; save the symbol to the RFIDBuffer
		clrf	Count					; clear Count
		clrf	ByteCount				; clear ByteCount
;		
SSP2_INT_Exit
		bcf		PIR3,		SSP2IF		; Clear the Interrupt
		return
;
;***************************************;***************************************
;
; 	Timer 0 Overflow
;
;	Changes
;		Flags.Tick		-	new tick (1/2 second)
;		INTCON.TMR0IF	-	clear the interrupt
;
;***************************************;***************************************
;		
TMR0_INT
		bsf		Flags,	Tick
;		
TMR0_INT_Exit
		bcf		INTCON,	TMR0IF
		return		
;
;***************************************;***************************************
;
; 	Initialization
;
;***************************************;***************************************
;
Init
Init_OSCCON
		banksel	OSCCON
		movlw	b'01110000'				; IDLEN=0, IRCD2:0=111,
										; OSTS=1, HFIOFS=0, SCS1:0=00
		movwf	OSCCON
		
		movlw	b'10000111'				; PLLRDY=1, SOSCRUN=0, x, MFIOSEL=0,
										; SOSCGO=0, PRISD=1, MFIOFS=1, LFIOFS=1
		movwf	OSCCON2	
		
		movlw	b'01000000'				; INTSRC=0, PLLEN=1, TUN5:0=00000
		movwf	OSCTUNE									
;
Init_PORTA
		banksel	ANSELA
		movlw	b'00000000'				; LED2=D, LED3=D, CS=D, LED1=D, 
										; x, x, x, DOOR=D
		movwf	ANSELA,		1
;	
	
		movlw	b'00000000'				; LED2=0, LED3=0, CS=0, LED1=0, 
										; x, x, x, DOOR=0
		movwf	PORTA
;		
		movlw	b'00001110'				; LED2=0, LED3=0, CS=0, LED1=0, 
										; x, x, x, DOOR=O
		movwf	TRISA
;
Init_PORTB
		bcf		INTCON2,	RBPU		; enable PORTB pullups
		movlw	b'00000000'				; CFG4:1=D,
										; MOD=D, DEMOD=D, RDY=D, SLP=D
		movwf	ANSELB,		1
;		
		movlw	b'11110111'				; CFG4:1=1111,
										; MOD=0, DEMOD=1, RDY=1, SLP=1
		movwf	PORTB
;		
		movlw	b'11110110'				; CFG4:1=1111,
										; MOD=0, DEMOD=1, RDY=1, SLP=0
		movwf	TRISB
;
Init_PORTC
		movlw	b'00000000'				; RX=D, TX=D, SDO=D, SDI=D,
										; SCK=D, LED6:4=D
		movwf	ANSELC,		1
;		
		movlw	b'11010000'				; RX=H, TX=H, SDO=L, SDI=H,
										; SCK=L, LED6:4=L
		movwf	PORTC
;		
		movlw	b'10010000'				; RX=I, TX=O, SDO=O, SDI=I,
										; SCK=O, LED6:4=O
		movwf	TRISC		
;	
Init_SPI2
		movlw	b'00000000'				; SMP=0, CKE=0, D/A=0, P=0,
										; S=0, R/W=0, UA=0, BF=0
		movwf	SSP2STAT
;
		movlw	b'00100100'				; WCOL=0, SSP2OV=0, SSP2EN=1, CKP=0,
										; SSP2M3:0=0100
		movwf	SSP2CON1
;
		movlw	b'00000000'				; ACKTIM=0, PCIE=0, SCIE=0, BOEN=0,
										; SDAHT=0, SBCDE=0, AHEN=0, DHEN=0
		movwf	SSP2CON3		
		movf	SSP2BUF,	W			; dummy read
		bcf		PIR3,		SSP2IF		; clear SPI2 Interrupts
		
;	
Init_UART
		movlw	8					
		movwf	SPBRG1					; 8=115200 (@64MHz)
;
		movlw	b'00100010'				; CSRC=0, TX9=0, TXEN=1, SYNC=0,
										; SENDB=0, BRGH=0, TRMT=1, TX9D=0		
		movwf	TXSTA1
;
		movlw	b'10010000'				; SPEN=1, RX9=0, SREN=0, CREN=1,
										; ADDEN=0, FERR=0, OERR=0, RX9D=0		
		movwf	RCSTA1
;
		bcf		PIR1,		RC1IF
;
Init_Timer0
		movlw	b'10000110'				; TMR0ON=1, T08BIT=0, T0CS=0, T0SE=0,
		movwf	T0CON					; PSA=0,  TOPS2:0=110
;
Init_Registers
		lfsr	1,	0					; use indirect pointer 1 for clear register loop
		movlw	6						; up through bank 6
;
Init_Registers_Loop		
		clrf	POSTINC1				; clear register
		cpfseq	FSR1H					; skip next if bank 6
		goto	Init_Registers_Loop		; go clear another		
;
		movlw	low TXBuffer			; initialize the TXBuffer
		movwf	TXBuffer_Head			; head and
		movwf	TXBuffer_Tail			; tail
		movlw	low RCBuffer			; initialize the RCBuffer
		movwf	RCBuffer_Head			; head and		
		movwf	RCBuffer_Tail			; tail		
;				
Init_Interrupts
		bsf		PIE1,		RC1IE		; enable UART
		bsf		PIE3,		SSP2IE		; enable SPI2
		bsf		INTCON,		TMR0IE		; enable TMR0		
;
		bsf		INTCON,		GIEL		; enable low priority
		bsf		INTCON,		GIEH		; enable high priority
;
		movlw	upper SignOn1			; Sigh On 1 message
		movwf	TBLPTRU
		movlw	high SignOn1
		movwf	TBLPTRH
		movlw	low SignOn1
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
		movlw	upper SignOn2			; Sigh On 2 message
		movwf	TBLPTRU
		movlw	high SignOn2
		movwf	TBLPTRH
		movlw	low SignOn2
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port	
;
		movlw	upper SignOn3			; Sigh On 3 message
		movwf	TBLPTRU
		movlw	high SignOn3
		movwf	TBLPTRH
		movlw	low SignOn3
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;			
        goto	Main 
;
;***************************************;***************************************
;
; 	MAIN PROGRAM Loop
;
;	Prerequisite
;		NoID				-	No ID message
;		Add					-	Add Codes message
;		Saved				- 	Saved message
;		AlreadySaved		- 	Already Saved message
;		Finished			- 	Finished message
;		Delete				-	Delete Codes message
;		Deleted				- 	Deleted message
;		AlreadyGone			- 	Already Gone message
;		UnlockCount			- 	Number of Tick (unlocked)
;		PORTB.4				-	CFG 1 - Add IDs
;		PORTB.5				-	CFG 2 - Remove IDs
;	Changed
;		Flags.RC1_In		-	none/character ready flag bit (receive)
;		RC1CharOut			-	character from serial receive buffer
;		TX1CharIn			-	character to serial transmit buffer
;		Flags.Timing		- 	none/in process flag bit (door unlock timer)
;		Flags.Tick			-	none/new tick flag bit (presently 1/2 second)
;		CodeOffset			-	ID offset
;		Count				- 	byte counter
;		Flags.BadCode		-	none/ID not found flag bit
;		EECHAR				- 	character to write to/read from EEPROM 
;		PRODL				-	calculated offset of required location in EEPROM
;		LATC.LED5			-	Add Codes LED
;		LATC.LED6			-	Delete Codes LED
;		UnlockCounter		- 	Unlock timer counter
;	Calls
;		Chk4RC1				- 	Check for serial reception
;		Chk4Command			-	act on received characters (not presently used)
;		Search				- 	look for an RFID card
;		Look4ID				-	look for an RFID card ID number
;		DisplayMatrix		- 	display the matrix to the serial port
;		DisplayID			-	display the ID to the serial port
;		Chk4Match			-	does this match anything in EEPROM
;		SendMessage			-	Send message to serial port
;		ReadEEPROM			-	read an EEPROM byte at location PRODL
;		WriteEEPROM			-	write the character into EEPROM at location PRODL
;
;***************************************;***************************************
;
Main
;
Main_Chk4RC1
		call	Chk4RC1					; check for serial reception
;
		btfss	Flags,	RC1_In			; skip next if we got a character
		goto	Main_Chk4RC1_Exit		; else done here
;
Main_RC1	
		movff	RC1CharOut,	TX1CharIn	; get character
		call	Chk4Command				; act on received characters (not presently used)
;		
Main_Chk4RC1_Exit
;				
;***************************************
;
Main_Chk4Search
		btfsc	Flags,	Timing			; skip next if door is NOT unlocked	
		goto	Main_Chk4Search_Exit	; else done here
;
		btfss	Flags,	Tick			; skip next if new tick (once per 1/2 second)
		goto	Main_Chk4Search_Exit	; else done here
;
		bcf		Flags,	Tick			; clear the tick
		call	Search					; look for an RFID card
;
		call	Look4ID					; look for an RFID card ID number
;
		btfsc	Flags,	NoCode			; skip next if we found an ID
		goto	Main_Chk4Search_NG		; else do No ID message
;
		call	DisplayMatrix			; display the matrix to the serial port
;
		call	DisplayID				; display the ID to the serial port
;		
		call	Chk4Match				; does this match anything in EEPROM
;
		goto	Main_Chk4Search_Exit	; done here
;
Main_Chk4Search_NG						; didn't find any ID
		movlw	upper NoID				; No ID message
		movwf	TBLPTRU
		movlw	high NoID
		movwf	TBLPTRH
		movlw	low NoID
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
Main_Chk4Search_Exit		
;
;***************************************
;		
Main_Chk4CFG1		
		btfsc	PORTB,	4				; skip next if CFG1 jumper is installed
		goto	Main_Chk4CFG1_Exit		; else done here
;
Main_Chk4CFG1_LED5_ON		
		bsf		LATC,	LED5			; light Add Codes LED
		movlw	upper Add				; Add message
		movwf	TBLPTRU
		movlw	high Add
		movwf	TBLPTRH
		movlw	low Add
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port		
;			
Main_Chk4CFG1_Loop
		btfss	Flags,	Tick			; skip next if new tick (once per 1/2 second)
		goto	Main_Chk4CFG1_Loop		; else wait loop
;		
		bcf		Flags,	Tick			; clear the tick
		call	Search					; look for an RFID card
;
		call	Look4ID					; look for an RFID card ID number
;
		btfsc	Flags,	NoCode			; skip next if we found an ID
		goto	Main_Chk4CFG1_Continue2
;
		call	DisplayMatrix			; display the matrix to the serial port
;
		call	DisplayID				; display the ID to the serial port
;		
		call	Chk4Match				; does this match anything in EEPROM
;
		btfss	Flags,	BadCode			; skip next if no match in EEPROM
		goto	Main_Chk4CFG1_AlreadyHere	; else its already in there, go on
;
Main_Chk4CFG1_NotHere					; we need to look for an empty entry location
		clrf	CodeOffset				; clear EEPROM offset
;
Main_Chk4CFG1_NotHere_IDLoop
		movlw 	5 						; number of bytes to get
		movwf	Count
		mulwf	CodeOffset				; times offset
;
Main_Chk4CFG1_NotHere_ByteLoop		
		call	ReadEEPROM				; read an EEPROM byte at
;
		setf	WREG
		cpfseq	EECHAR					; skip next if EECHAR=0xFF (empty)
		goto	Main_Chk4CFG1_NotHere_NOMatch	; else no match
;
Main_Chk4CFG1_NotHere_Match
		dcfsnz	Count,	F				; skip next if Count-1<>0		
		goto	Main_Chk4CFG1_NotHere_IsEmpty	; else we found 5 empty locations, go on
;
		incf	PRODL,	F				; increment to the next location
		goto	Main_Chk4CFG1_NotHere_ByteLoop	; go read another location
;
Main_Chk4CFG1_NotHere_NOMatch
		incfsz	CodeOffset				; not empty so move to the start of the next ID
		goto	Main_Chk4CFG1_NotHere_IDLoop	; go start another ID read
;
Main_Chk4CFG1_NotHere_IsEmpty			; now save the ID into the empty slot
		lfsr	0,	RFIDIDBuffer		; use indirect pointer 0 for RFIDIDBuffer buffer
		movlw	5
		movwf	Count					; 5 byes in an ID
		mulwf	CodeOffset				; times the (empty) ID offset 
;
Main_Chk4CFG1_NotHere_Loop
		movff	POSTINC0,	EECHAR		; get an ID byte
		call	WriteEEPROM				; write the character into EEPROM at location PRODL 
;
		dcfsnz	Count					; skip next if Count-1<>0
		goto	Main_Chk4CFG1_Continue	; else we must be done, go on
;
		incf	PRODL,	F				; increment to the next location
		goto	Main_Chk4CFG1_NotHere_Loop	; go get another ID byte	
;
Main_Chk4CFG1_AlreadyHere				
		movlw	upper AlreadySaved		; Already Saved message
		movwf	TBLPTRU
		movlw	high AlreadySaved
		movwf	TBLPTRH
		movlw	low AlreadySaved
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
		goto	Main_Chk4CFG1_Continue2	; go on
;
Main_Chk4CFG1_Continue
		movlw	upper Saved				; Saved message
		movwf	TBLPTRU
		movlw	high Saved
		movwf	TBLPTRH
		movlw	low Saved
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port	
;
Main_Chk4CFG1_Continue2					
		btfss	PORTB,	4				; skip next if CFG1 jumper is removed				
		goto	Main_Chk4CFG1_Loop		; else look for more IDs to add		
;
Main_Chk4CFG1_LED5_OFF
		movlw	upper Finished			; Finished message
		movwf	TBLPTRU
		movlw	high Finished
		movwf	TBLPTRH
		movlw	low Finished
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;		
		bcf		LATC,	LED5			; Add Codes LED OFF
;				
Main_Chk4CFG1_Exit	
;			
;***************************************
;		
Main_Chk4CFG2		
		btfsc	PORTB,	5				; skip next if CFG2 jumper is removed
		goto	Main_Chk4CFG2_Exit		; done here
;
Main_Chk4CFG2_LED6_ON		
		bsf		LATC,	LED6			; Light Delete Codes LED
		movlw	upper Delete			; Delete message
		movwf	TBLPTRU
		movlw	high Delete
		movwf	TBLPTRH
		movlw	low Delete
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
Main_Chk4CFG2_Loop
		btfss	Flags,	Tick			; skip next if new tick (once per 1/2 second)
		goto	Main_Chk4CFG2_Loop		; else wait loop
;		
		bcf		Flags,	Tick			; clear the tick
		call	Search					; look for an RFID card
;
		call	Look4ID					; look for an RFID card ID number
;
		btfsc	Flags,	NoCode			; skip next if we found an ID
		goto	Main_Chk4CFG2_Continue2
;
		call	DisplayMatrix			; display the matrix to the serial port
;
		call	DisplayID				; display the ID to the serial port
;
		call	Chk4Match				; does this match anything in EEPROM
;
		btfss	Flags,	BadCode			; skip next if ID NOT in EEPROM
		goto	Main_Chk4CFG2_Here		; else we go it, go on
;
Main_Chk4CFG2_NotHere
		movlw	upper AlreadyGone		; Already Gone message
		movwf	TBLPTRU
		movlw	high AlreadyGone
		movwf	TBLPTRH
		movlw	low AlreadyGone
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
		goto	Main_Chk4CFG2_Continue2	; go on
;
Main_Chk4CFG2_Here						; we found the ID to delete
		movlw	5
		movwf	Count					; 5 byte to delete
		mulwf	CodeOffset				; ID offset
;
Main_Chk4CFG2_Here_Loop
		setf	EECHAR					; erase by writing 0xFF
		call	WriteEEPROM				; write the character into EEPROM at location PRODL	
;
		decfsz	Count					; skip next if Count-1<>0
		goto	Main_Chk4CFG2_Here_Done	; else done erasing, go on	
;
		incf	PRODL,	F				; increment location
		goto	Main_Chk4CFG2_Here_Loop	; erase another byte location	
;;
Main_Chk4CFG2_Here_Done
		movlw	upper Deleted			; Deleted message
		movwf	TBLPTRU
		movlw	high Deleted
		movwf	TBLPTRH
		movlw	low Deleted
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port	
;
Main_Chk4CFG2_Continue2
		btfss	PORTB,	5				; skip next if CFG2 jumper is removed
		goto	Main_Chk4CFG2_Loop		; else look for more IDs to delete
;
Main_Chk4CFG2_LED6_OFF
		movlw	upper Finished			; Finished message
		movwf	TBLPTRU
		movlw	high Finished
		movwf	TBLPTRH
		movlw	low Finished
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port	
;
		bcf		LATC,	LED6			; Add Codes LED OFF
	
Main_Chk4CFG2_Exit		
;		
;***************************************
;
Main_Chk4Good
		btfss	Flags,	GoodCode		; skip next if we found a Good Code
		goto	Main_Chk4Good_Exit		; else done here
;		
		btfsc	Flags,	Timing			; skip next if unlock timer is OFF
		goto	Main_Chk4Good_Exit		; else done here
;		
Chk4Match_UnlockMessage
		bsf		LATC,	LED4			; Light Unlocked Door LED
		bsf		LATA,	DOOR			; unlock door
		movlw	upper Unlock			; Unlock message	
		movwf	TBLPTRU
		movlw	high Unlock
		movwf	TBLPTRH
		movlw	low Unlock
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;		
		movlw	UnlockCount				; 
		movwf	UnlockCounter			; load the unlock timer counter		 
		bsf		Flags,	Timing			; start the timer
;
Main_Chk4Good_Exit
;
;***************************************
;
Main_Chk4Bad
		btfss	Flags,	BadCode			; skip next if we found a Good Code
		goto	Main_Chk4Bad_Exit		; else done here
;		
		btfsc	Flags,	Timing			; skip next if unlock timer is OFF
		goto	Main_Chk4Bad_Exit		; else done here
;		
		movlw	UnlockCount
		movwf	UnlockCounter			; load the unlock timer counter
		bsf		Flags,	Timing			; start the timer
;
Main_Chk4Bad_Exit
;***************************************
Chk4Match_Timing	
		btfss	Flags,	Timing			; skip next if unlock timer is timing
		goto	Chk4Match_Timing_Exit		; else done here
;		
		btfss	Flags,	Tick			; skip next if new Tick
		goto	Chk4Match_Timing_Exit	; else done here
;
		bcf		Flags,	Tick			; clear new Tick
		decfsz	UnlockCounter,	F		; skip next if UnlockCounter-1=0
		goto	Chk4Match_Timing_Exit	; else done here
;
		bcf		Flags,	Timing			; stop the timer
		btfss	Flags,	GoodCode		; skip next if we have a Good Code
		goto	Chk4Match_Timing_BadCode	; else Bad Code, go on
;
Chk4Match_Timing_GoodCode				
		movlw	upper Lock				; Lock message
		movwf	TBLPTRU
		movlw	high Lock
		movwf	TBLPTRH
		movlw	low Lock
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;		
		bcf		LATC,	LED4			; Unlocked Door LED OFF
		bcf		LATA,	DOOR			; lock the door
		bcf		Flags,	GoodCode		; clear the Good Code
		bcf		LATA,	LED1			; Good Code LED OFF
		goto	Chk4Match_Timing_Exit	; go on		
;
Chk4Match_Timing_BadCode	
		bcf		Flags,	BadCode			; clear the Bad Code
		bcf		LATA,	LED2			; Bad Code LED OFF
;						
Chk4Match_Timing_Exit	
;				
;***************************************
;
        goto	Main                    ; end of main loop 
;
;***************************************;***************************************
;
;	Check for any Command				; presently not used
;
;	S 	=	Start a search
;
;***************************************;***************************************
;        
Chk4Command
;		movlw	a'S'
;		cpfseq	RC1CharOut				; skip next if Commnand S
;		goto	Chk4Command_Exit
;
Chk4Command_S
;		
Chk4Command_Exit			
		return
;
;***************************************;***************************************
;
;	Search
;
;	Prerequisite
;		Searching			- 	Searching message
;		Finished			- 	Finished message
;	Changed
;		TBLPTRU:H:L			-	code message pointer
;		LATA.LED3			-	Seraching LED
;		LATB.SLP			- 	Sleep control output
;		FSR0				-	RFIDBuffer Buffer
;	Calls
;		SendMessage			-	Send message to serial port
;
;***************************************;***************************************
;		
Search	
		bsf		LATA,	LED3			; light the seaching LED
		movlw	upper Searching			; pointer to Searching message
		movwf	TBLPTRU
		movlw	high Searching
		movwf	TBLPTRH
		movlw	low Searching
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;		
		lfsr	0,		RFIDBuffer		; use indirect pointer 0 for RFIDBuffer
		bcf		LATB,	SLP				; Start the search (enabled SPI2}
;
Search_Loop
		movlw	3
		cpfseq	FSR0H					; skip next if RFID Buffer is full (0x300)
		goto	Search_Loop				; else try another
;
		bsf		LATB,	SLP				; Stop the search (disabled SPI2}
		movlw	upper Finished			; pointer to Finished message
		movwf	TBLPTRU
		movlw	high Finished
		movwf	TBLPTRH
		movlw	low Finished
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
		bcf		LATA,	LED3			; seaching LED off		
		return	
;
;***************************************;***************************************
;
;	Read a byte from the EEPROM
;
;	Prerequisite
;		PRODL			- 	offset calculation for destination
;	Changed
;		EEADR			-	EEPROM address
;		EECON1.EEPGD	-	Date/Code	
;		EECON1.CFGS		-	enable/disable access
;		EECON1.RD		-	enable/disable read 
;		EEDATA			-	EEPROM data
;		EECHAR			-	byte read
;	Calls
;		none
;
;***************************************;***************************************
;		
ReadEEPROM		
		movff	PRODL,	EEADR			; Data Memory Address to read
		bcf		EECON1, EEPGD 			; Point to DATA memory
		bcf		EECON1, CFGS 			; Access EEPROM
		bsf		EECON1, RD 				; EEPROM Read
		movff	EEDATA, EECHAR			; EEChar = EEDATA
		return	
;
;***************************************;***************************************
;
;	Write a byte from the EEPROM
;
;	Prerequisite
;		PRODL			- 	offset calculation for destination
;		EECHAR			-	byte to write
;	Changed
;		EEADR			-	EEPROM address
;		EEDATA			-	EEPROM data
;		EECON1.EEPGD	-	Date/Code	
;		EECON1.CFGS		-	enable/disable access
;		EECON1.WREN		-	enable/disable write (protection) 
;		INTCON.GIE		-	enable/disable global interrupt
;		EECON2			-	write protection register
;	Calls
;		none
;
;***************************************;***************************************
;
WriteEEPROM		
		movff 	PRODL,	EEADR 			; Data Memory Address to write
		movff 	EECHAR,	EEDATA 			; Data Memory Value to write
		bcf 	EECON1,	EEPGD 			; Point to DATA memory
		bcf 	EECON1, CFGS 			; Access EEPROM
		bsf 	EECON1, WREN 			; Enable writes
		bcf 	INTCON, GIE 			; Disable Interrupts
		movlw 	55h 					;
		movwf 	EECON2 					; Write 55h
		movlw 	0AAh 					;		
		movwf 	EECON2 					; Write 0AAh
		bsf 	EECON1, WR 				; Set WR bit to begin write
;
WriteEEPROM_Wait		
		btfsc 	EECON1, WR 				; Wait for write to complete
		goto 	WriteEEPROM_Wait
;	
		bcf 	EECON1, WREN 			; Disable writes on write complete (EEIF set)		
		bsf 	INTCON, GIE 			; Enable Interrupts
		return			
;
;***************************************;***************************************
;
;	Try and make sense out of the RFID Buffer
;
;	Prerequisite
;		none
;	Changed
;		Flags.NoCode	-	None/GotGoodCode flag bit (no code)	
;		FSR0			-	RFIDBuffer Buffer
;		Mode			-	state 
;		Flags.ODD		-	EVEN/ODD flag bit
;		TX1CharIn		-	transfer variable
;		EvenData		-	value of the EVEN byte (symbol pair, EVEN ODD bytes)
;		HeaderCount		-  	header byte count
;		IDCount			-	ID byte counter
;		Temp1			-	temporary variable
;	Calls
;		none
;
;***************************************;***************************************
;		
Look4ID	
		bcf		Flags,	NoCode			; initialize NoCode flag bit as none
		lfsr	0,		RFIDBuffer		; use indirect pointer 0 as the search buffer
		clrf	Mode					; clear the state
		bcf		Flags,	ODD				; first byte is assumed to be an even byte
;
Look4ID_Loop
		movff	POSTINC0,	TX1CharIn	; get a character from the buffer
;		
		movlw	3
		cpfseq	FSR0H					; skip next if past the RFID Buffer (0x300)
		goto	Look4ID_Chk4Mode0		; else go on
;		
Look4ID_Done		
		goto	Look4ID_NG				; we didn't find anything
;		
Look4ID_Chk4Mode0						
		movlw	0
		cpfseq	Mode					; skip next if mode=0
		goto	Look4ID_Chk4Mode1		; else go on
;
Look4ID_Mode0							; look for the header
		btfsc	Flags,	ODD				; skip next if this byte is EVEN
		goto	Look4ID_Mode0_ODD		; else its odd (second in pair)
;
Look4ID_Mode0_EVEN
		movff	TX1CharIn,	EvenData	; save EvenData
		bsf		Flags,	ODD				; next is an ODD byte
		goto	Look4ID_Loop			; go get another
;					
Look4ID_Mode0_ODD
		clrf	WREG	
		cpfseq	TX1CharIn				; skip next if odd byte=0
		goto	Look4ID_Mode0_1			; else odd byte=1 	
;		
Look4ID_Mode0_0
		movf	EvenData,	W			; get EvenData
		cpfseq	TX1CharIn				; skip next if the odd and even bytes are same 
		goto	Look4ID_Mode0_10		; else they are the different
;
Look4ID_Mode0_00						; illegal
		goto	Look4ID_Loop			; go try another		
;
Look4ID_Mode0_10						; a header byte
		movlw	1
		movwf	Mode					; next time mode =1
		incf	HeaderCount				; we've got 1 header byte in a row
		lfsr	1,		RFIDDataBuffer	; use indirect pointer 1 as RFIDDataBuffer buffer
		movlw	0x31
		movwf	POSTINC1				; save an ASCII 1 (header byte)
		bcf		Flags,	ODD				; next byte is even
		goto	Look4ID_Loop			; go get another
;
Look4ID_Mode0_1
		movf	EvenData,	W			; get EvenData
		cpfseq	TX1CharIn				; skip next if the odd and even bytes are same 
		goto	Look4ID_Mode0_01		; else they are the different
;
Look4ID_Mode0_11						; illegal
		goto	Look4ID_Loop			; go try another
;		
Look4ID_Mode0_01						; not a header byte
		bcf		Flags,	ODD				; next byte is even
		goto	Look4ID_Loop			; go try another

Look4ID_Chk4Mode1
		movlw	1
		cpfseq	Mode					; skip next if mode=1
		goto	Look4ID_Chk4Mode2		; else go on
;
Look4ID_Mode1							; retrieve the rest of the header
		btfsc	Flags,	ODD				; skip next if the byte is even (first in pair)
		goto	Look4ID_Mode1_ODD		; else its odd (second in pair)
;
Look4ID_Mode1_EVEN
		movff	TX1CharIn,	EvenData	; save EvenData
		setf	WREG					
		cpfseq	TX1CharIn				; skip next if the byte=1 
		goto	Look4ID_Mode1_EVEN0		; else it must be '0' (should be a '1')
;		
Look4ID_Mode1_EVEN1
		bsf		Flags,	ODD				; next byte will be odd
		goto	Look4ID_Loop			; go get another
;
Look4ID_Mode1_EVEN0						; illegal (go start over)
		goto	Look4ID_Error			; this is an error
;
Look4ID_Mode1_ODD
		clrf	WREG			
		cpfseq	TX1CharIn				; skip next if the byte=0 
		goto	Look4ID_Mode1_1			; else it must be '1' (should be a '0')	
;		
Look4ID_Mode1_0	
		movf	EvenData,	W
		cpfseq	TX1CharIn				; skip next if odd and even bytes are the same
		goto	Look4ID_Mode1_10
;
Look4ID_Mode1_00						; illegal (go start over)
		goto	Look4ID_Error			; this is an error
;
Look4ID_Mode1_10
		bcf		Flags,	ODD				; next byte will be even
		incf	HeaderCount				; increment the Header Count	
		movlw	0x31
		movwf	POSTINC1				; save the Header byte (ASCII 1)
		movlw	9
		cpfseq	HeaderCount				; skip next if we have all the Header bytes
		goto	Look4ID_Loop			; else go get another
;
		movlw	2
		movwf	Mode					; mode=2				
		bcf		Flags,	ODD				; next byte will be even			
		clrf	IDCount					; IDCount=0
		goto	Look4ID_Loop			; go get another				
;
Look4ID_Mode1_1
		movf	EvenData,	W			; get EvenData
		cpfseq	TX1CharIn				; skip next if odd and even bytes are the same
		goto	Look4ID_Mode1_01		; else they are different
;
Look4ID_Mode1_11						; illegal (go start over)
		goto	Look4ID_Error			; this is an error
;
Look4ID_Mode1_01						; illegal, not a header byte (go start over)
		goto	Look4ID_Error			; this is an error		
;
Look4ID_Chk4Mode2				
		movlw	2
		cpfseq	Mode					; skip next if Mode=2
		goto	Look4ID_NG				; else this is an error
;
Look4ID_Mode2							; retrieve data
		btfsc	Flags,	ODD				; skip next if the byte is even (first of pair)
		goto	Look4ID_Mode2_ODD		; else its odd (second of pair)
;
Look4ID_Mode2_EVEN
		movff	TX1CharIn,	EvenData	; save EvenData
		bsf		Flags,	ODD				; next byte will be odd	
		goto	Look4ID_Loop			; go get another		
;
Look4ID_Mode2_ODD
		setf	WREG
		cpfseq	TX1CharIn				; skip next if it is a '1'
		goto	Look4ID_Mode2_0		
;
Look4ID_Mode2_1
		movf	EvenData,	W			; get EvenData
		cpfseq	TX1CharIn				; skip next if odd and even bytes are the same
		goto	Look4ID_Mode2_01		; else the bytes are different
;
Look4ID_Mode2_11						; illegal (go start over)
		goto	Look4ID_Error			; this is an error
;		
Look4ID_Mode2_01
		movlw	0x30
		movwf	TX1CharIn				; get ASCII 0
		goto	Look4ID_Mode2_Continue	; go on			
;
Look4ID_Mode2_0
		movf	EvenData,	W			; get EvenData
		cpfseq	TX1CharIn				; skip next if odd and even bytes are the same
		goto	Look4ID_Mode2_10		; else the bytes are different
;
Look4ID_Mode2_00						; illegal (go start over)
		goto	Look4ID_Error			; this is an error
;
Look4ID_Mode2_10
		movlw	0x31
		movwf	TX1CharIn				; get ASCII 1
;
Look4ID_Mode2_Continue
		movff	TX1CharIn,	POSTINC1	; save the ASCII data
		bcf		Flags,	ODD				; next byte will be even
		incf	IDCount,	F			; increment the IDCount (data)
		movlw	D'55'
		cpfseq	IDCount					; skip next if got it all
		goto	Look4ID_Loop			; else go get another
;
		goto	Look4ID_Exit			; all done here
;
Look4ID_Error
		clrf	Mode					; Mode=0
		clrf	HeaderCount				; HeaderCount=0
		goto	Look4ID_Loop			; start over again					
;			
Look4ID_NG
		bsf		Flags,	NoCode			; we didn't find any legal Code
;			
Look4ID_Exit
		return
;
;***************************************;***************************************
;
;	Display Matrix
;
;	Prerequisite
;		Matrix				- 	Matrix message
;	Changed
;		TBLPTRU:H:L			-	code message pointer
;		HeaderCount			-  	header byte count
;		FSR0				-	RFIDDataBuffer Buffer
;		TX1CharIn			-	transfer variable
;		IDCount				-	ID byte counter
;	Calls
;		AddCharToTXBuffer	-	Put TX1CharIn into TXBuffer
;		SendMessage			-	Send message to serial port
;
;***************************************;***************************************
;				
DisplayMatrix
		movlw	upper Matrix			; point to Matrix message
		movwf	TBLPTRU
		movlw	high Matrix
		movwf	TBLPTRH
		movlw	low Matrix
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
		movlw	9
		movwf	HeaderCount				; HeaderCount=9 header bytes
		lfsr	0,	RFIDDataBuffer		; use indirect pointer 0 for RFIDDataBuffer
;
DisplayMatrix_HeaderLoop
		movff	POSTINC0,	TX1CharIn
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer	
;
		movlw	0x20
		movwf	TX1CharIn
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer		
;
		decfsz	HeaderCount				; skip next if HeaderCount-1=0
		goto	DisplayMatrix_HeaderLoop	; else more to do				
;
		movlw	0x0D
		movwf	TX1CharIn				; get CR
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	0x0A
		movwf	TX1CharIn				; get LF
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	0x0B
		movwf	IDCount					; IDCount=11 groups (10nibbles+1parity)
;
DisplayMatrix_IDLoop
		movlw	5
		movwf	HeaderCount				; HeaderCount = 5 bytes (4bits+1parity)
;
DisplayMatrix_BitLoop
		movff	POSTINC0,	TX1CharIn	; get next byte
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	0x20
		movwf	TX1CharIn				; get ASCII <space>
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer		
;
		decfsz	HeaderCount				; skip next if HeaderCount-1=0			
		goto	DisplayMatrix_BitLoop	; else more to do
;
		movlw	0x0D
		movwf	TX1CharIn				; get CR
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	0x0A
		movwf	TX1CharIn				; get LF 
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		decfsz	IDCount					; skip next if IDCount-1=0
		goto	DisplayMatrix_IDLoop	; else more to do
;
		movlw	0x0D
		movwf	TX1CharIn				; get CR
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	0x0A
		movwf	TX1CharIn				; get LF
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
DisplayMatrix_Exit
		return		
;
;***************************************;***************************************
;
;	Display ID
;
;	Prerequisite
;		Parity				- 	Parity error message
;	Changed
;		Flags.NoCode		-	None/GotGoodCode flag bit (no code)
;		TX1CharIn			-	transfer variable
;		FSR0				-	RFIDDataBuffer Buffer 
;		FSR1				-	RFIDIDBuffer Buffer
;		IDCount				-	ID byte counter
;		ColumnParity		- 	parity varaible
;		RowParity			- 	parity varaible
;		Temp1				-	temporary variable
;		Temp2				-	temporary variable
;		HeaderCount			-  	header byte count
;	Calls
;		AddCharToTXBuffer	-	Put TX1CharIn into TXBuffer
;		SendMessage			-	Send message to serial port
;		HEX2ASCII			-	convert a value to 2 ASCII characters
;
;***************************************;***************************************
;				
DisplayID
		bcf		Flags,	NoCode			; initialize NoCode flag bit as none
		movlw	a'I'
		movwf	TX1CharIn				; ASCII I
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	a'D'
		movwf	TX1CharIn				; ASCII D
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	':'
		movwf	TX1CharIn				; ASCII <colon>			
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	0x20
		movwf	TX1CharIn				; ASCII <space>
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer		
;
		lfsr	0,	RFIDDataBuffer+9	; use indirect pointer 0 for RFIDDataBuffer
		lfsr	1,	RFIDIDBuffer		; use indirect pointer 1 for RFIDIDBuffer
		movlw	0x0A	
		movwf	IDCount					; IDCount = 10 bytes
		clrf	ColumnParity			; clear column parity
;
DisplayID_IDLoop
		movlw	4
		movwf	HeaderCount				; 4 bits for row parity
		clrf	RowParity				; clear row parity
		clrf	Temp1					; clear the temporary parity variable 
;
DisplayID_BitLoop
		movff	POSTINC0,	TX1CharIn	; get next byte
		rrcf	TX1CharIn				; rotate right parity bit out				; 
		rlcf	Temp1					; rotate left parity bit in
		btfsc	Temp1,	0				; skip next if parity is even
		incf	RowParity,	F			; else its odd, invert the parity bit
;
		decfsz	HeaderCount				; skip next if HeaderCount-1=0
		goto	DisplayID_BitLoop		; else go do more bits
;		
		movff	POSTINC0,	TX1CharIn	; get next byte
		btfsc	TX1CharIn,	0			; skip next if parity is even
		goto	DisplayID_BitLoop_PO	; else parity is off, go on
;
DisplayID_BitLoop_PE
		btfsc	RowParity,	0			; skip next if row parity is even
		goto	DisplayID_BitLoop_PE_RO	; else its odd, go on
;
DisplayID_BitLoop_PE_RE					; parity match
		goto	DisplayID_BitLoop_Continue	; go on
;
DisplayID_BitLoop_PE_RO					; parity mismatch
		goto	DisplayID_ParityError	; go to error
;
DisplayID_BitLoop_PO
		btfsc	RowParity,	0			; skip next if row parity is even
		goto	DisplayID_BitLoop_PO_RO	; else its odd, go on
;
DisplayID_BitLoop_PO_RE					; parity mismatch
		goto	DisplayID_ParityError	; go to error
;
DisplayID_BitLoop_PO_RO					; parity match
		goto	DisplayID_BitLoop_Continue	; go on
;
DisplayID_ParityError
		bsf		Flags,	NoCode			; NoCode flag bit
		movlw	upper Parity			; parity message
		movwf	TBLPTRU
		movlw	high Parity
		movwf	TBLPTRH
		movlw	low Parity
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;		
		goto	DisplayID_Exit			; done here
;
DisplayID_BitLoop_Continue
		btfss	IDCount,	0			; skip next if second symbol
		goto	DisplayID_BitLoop_FirstSymbol	; else first symbol, go on
;
DisplayID_BitLoop_SecondSymbol
		movf	Temp1,	W				;
		andlw	0x0F					; we want just the lower nibble
		iorwf	Temp2,	F				; or with the upper nibble
		movff	Temp2,	POSTINC1		; save to buffer
		goto	DisplayID_BitLoop_Continue1		; go on
;
DisplayID_BitLoop_FirstSymbol				
		swapf	Temp1,	W				; 
		andlw	0xF0					; we want just the upper nibble
		movwf	Temp2					; save it
;
DisplayID_BitLoop_Continue1			
		movf	ColumnParity,	W		; get column parity
		xorwf	Temp1,	W				; exclusive or with last data 
		movwf	ColumnParity			; save new parity
;
		decfsz	IDCount					; skip next if IDCount-1=0
		goto	DisplayID_IDLoop		; else more to do
;
		movlw	4						;
		movwf	HeaderCount				; 4 bits for row parity
		clrf	RowParity				; clear RowParity
		clrf	Temp1					; clear the temporary parity variable
;
DisplayID_ParityLoop
		movff	POSTINC0,	TX1CharIn	; get next character
		rrcf	TX1CharIn,	F			; rotate right parity bit out
		rlcf	Temp1,	F				; rotate left parity bit in
;
		decfsz	HeaderCount				; skip next if HeaderCount-1=0
		goto	DisplayID_ParityLoop	; else more to do
;
		movff	POSTINC0,	TX1CharIn	; (should be zero)
		movf	ColumnParity,	W		; get column parity
		cpfseq	Temp1					; skip next if parity match
		goto	DisplayID_ParityError	; else its a parity error		
;
DisplayID_Continue				
		lfsr	0,	RFIDIDBuffer		; use indirect pointer 0 for RFIDIDBuffer
		movlw	0x05
		movwf	IDCount					; 5 byte to send	
;
DisplayID_Loop				
		movff	POSTINC0,	Temp1		; get an IDbyte
		call	HEX2ASCII				; convert a value to 2 ASCII characters 
;
		movff	Temp2,	TX1CharIn		; upper ASCII character
		movff	Temp1,	Temp2			; save lower
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movff	Temp2,	TX1CharIn		; lower ASCII character		
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer		
;
		decfsz	IDCount					; skip next if IDCount-1=0
		goto	DisplayID_Loop			; else go do another ID byte
;
		movlw	0x0D
		movwf	TX1CharIn				; get CR
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
		movlw	0x0A
		movwf	TX1CharIn				; get LF
		call	AddCharToTXBuffer		; Put TX1CharIn into TXBuffer
;
DisplayID_Exit
		return
;
;***************************************;***************************************
;
;	Check for a match with what's in the EEPROM
;
;	Prerequisite
;		NoMatch			-	No match message
;		Match			-	Match message
;	Changed
;		Flags.GoodCode	-	None/GotGoodCode flag bit (match)
;		Flags.BadCode	-	None/GotBadCode flag bit (no match)
;		Flags.NoCode	-	None/GotGoodCode flag bit (no code)
;		CodeOffset		-	Code offset into EEPROM
;		Count			-	byte counter  
;		FSR0			-	Code ID Buffer 
;		EECHAR			-	byte read from EEPROM
;		PRODH:L			- 	offset calculation
;		LATA.LED2		-	Bad match LED
;		LATA.LED1		-	Good match LED
;	Calls
;		ReadEEPROM		-	Read character at PRODL in EEPROM
;		SendMessage		-	Send message to serial port
;
;***************************************;***************************************
; 		
Chk4Match
		bcf		Flags,	GoodCode		; initialiaze none
		bcf		Flags,	BadCode			; initialiaze none
		bcf		Flags,	NoCode			; initialiaze none
		clrf	CodeOffset				; zero offset
;
Chk4Match_IDLoop
		movlw 	5 						; number of bytes to get
		movwf	Count				
		mulwf	CodeOffset				; times offset
		lfsr 	0,	RFIDIDBuffer		; use indirect pointer 0 for RFIDIDBuffer
;
Chk4Match_ByteLoop	
		call	ReadEEPROM				; Read character at PRODL in EEPROM
;
		movf	EECHAR,	W				; character from EEPROM
		cpfseq	POSTINC0				; skip next if match with RFIDIDBuffer
		goto	Chk4Match_NOByteMatch	; else no match, go on
;
Chk4Match_ByteMatch
		dcfsnz	Count,	F				; skip next if Count-1<>0
		goto	Chk4Match_DoneMatch		; else we are done, go on
;
		incf	PRODL,	F				; next offset into EEPROM
		goto	Chk4Match_ByteLoop		; go get another character
;
Chk4Match_NOByteMatch
		incf	CodeOffset				; CodeOffset=CodeOffset+1
		movlw	d'25'
		cpfsgt	CodeOffset				; skip next if CodeOffset>25
		goto	Chk4Match_IDLoop		; else go try another ID
;
Chk4Match_DoneNOMatch					
		bsf		LATA,	LED2			; light No Match LED
		bsf		Flags,	BadCode			; flag Bad Code	
		movlw	upper NoMatch			; point to message
		movwf	TBLPTRU
		movlw	high NoMatch
		movwf	TBLPTRH
		movlw	low NoMatch
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
		goto	Chk4Match_Exit
;
Chk4Match_DoneMatch
		bsf		LATA,	LED1			; light Match LED
		bsf		Flags,	GoodCode		; flag Good Code
		movlw	upper Match				; point to message
		movwf	TBLPTRU
		movlw	high Match
		movwf	TBLPTRH
		movlw	low Match
		movwf	TBLPTRL				
		call	SendMessage				; Send message to serial port
;
;***************************************
Chk4Match_Exit
		return		
;
;***************************************;***************************************
;
;	Check for any characters in the RC1 Buffer
;
;	Prerequisite
;		Flags.RC1_In	-	NoChar/GotOne flag bit
;		RCBuffer_Size	-	RCBuffer size
;	Changed
;		INTCON.GIEH		-	Enable/disable global (high priority) interrupts
;		FSR1			-	RCBuffer
;		RC1CharOut		-	transfer variable
;		RCBuffer_Tail	-	RCBuffer pointer (removal)
;	Calls
;		none
;
;***************************************;***************************************
;  
Chk4RC1 
		bcf		INTCON,	GIEH			; disable global interrupts
		bcf		Flags,	RC1_In			; initialize as NoCharacter available
		movf	RCBuffer_Head,	W		; get head pointer
		cpfseq	RCBuffer_Tail			; skip next if same (TXBuffer is empty)
		goto	Chk4RC1_Char			; else go get the next character
;
Chk4RC1_Empty
		goto	Chk4RC1_Exit			; done here (no character)
;
Chk4RC1_Char
		lfsr	1,	RCBuffer			; RCBuffer 
		movff	RCBuffer_Tail,	FSR1L	; pointer (removal)
		movff	POSTINC1,	RC1CharOut	; get character from buffer
		movlw	low RCBuffer			; start of RCBuffer
		addlw	RCBuffer_Size			; add size of buffer
		cpfseq	FSR1L					; skip next if RCBuffer Overflow
		goto	Chk4RC1_RCBufferOK		; no overflow, so go on
;
Chk4RC1_RCBufferOVF
		movlw	low	RCBuffer			; start of RCBuffer
		movwf	FSR1L					; new pointer location
;
Chk4RC1_RCBufferOK		
		movff	FSR1L,	RCBuffer_Tail	; save new pointer
		bsf		Flags,	RC1_In			; Character available flag bit 
;
Chk4RC1_Exit
		bsf		INTCON,	GIEH			; enable global interrupts				
		return       
;
;***************************************;***************************************
;
;	Send Message
;
;	Prerequisite
;		TBLPTRU:H:L			-	Message pointer
;	Changed
;		TABLAT				- 	Message character
;		TX1CharIn			-	transfer variable
;	Calls
;		AddCharToTXBuffer	-	Put TX1CharIn into TXBuffer
;
;***************************************;***************************************
;        
SendMessage
		TBLRD*+							; get a character from the table
		movlw	0
		cpfseq	TABLAT					; skip next if Char=0
		goto	SendMessage_Char		; else we got more to display
;
		goto	SendMessage_CRLF		; go add CR and LF				
;
SendMessage_Char		
		movff	TABLAT,	TX1CharIn		; get character 
		call	AddCharToTXBuffer		; put it into the TXBuffer
;		
		goto	SendMessage				; get another
;
SendMessage_CRLF
		movlw	CR
		movwf	TX1CharIn				; get CR
		call	AddCharToTXBuffer		; put it into the TXBuffer
;		
		movlw	LF
		movwf	TX1CharIn				; get LF
		call	AddCharToTXBuffer		; put it into the TXBuffer
;
SendMessage_Exit						
		return 
;
;***************************************;***************************************
;
;	HEX to ASCII
;
;	Prerequisite
;		Temp1	-	Value
;	Changed
;		Temp1	- 	ASCII (lower nibble)
;		Temp2	-	ASCII (upper nibble
;	Calls
;		none
;
;***************************************;***************************************
;        
HEX2ASCII
		swapf	Temp1,	W				; get upper nibble 
		andlw	0x0F					; mask off upper
		iorlw	0x30					; make it ASCII
		movwf	Temp2					; save it
		movlw	0x39					;
		cpfsgt	Temp2					; skip next if its > ASCII 9
		goto	HEX2ASCII_1				; else its OK the way it is, go on
;
		movlw	7						; offset
		addwf	Temp2,	F				; add the offset to make it ASCII A:F
;
HEX2ASCII_1
		movf	Temp1,	W				; get the lower nibble
		andlw	0x0F					; mask off upper
		iorlw	0x30					; make it ASCII
		movwf	Temp1					; save it
		movlw	0x39					;
		cpfsgt	Temp1					; skip next if its > ASCII 9
		goto	HEX2ASCII_Exit			; else its OK the way it is, go on
;
		movlw	7						; offset
		addwf	Temp1,	F				; add the offset to make it ASCII A:F
;
HEX2ASCII_Exit
		return		
;
;***************************************;***************************************
;
;	Add a Character to the TXBuffer
;
;	Prerequisite
;		TX1CharIn		-	Character to Add
;	Changed
;		FSR1			-	TX Buffer 
;		PIE.TX1IE		-	Enable/disable Transmit interrupt
;		Temp1			-	temporary value
;		TXBuffer_Head	-	pointer
;		INTCON.GIEH		-	Enable/disable global (high priority) interrupts
;	Calls
;		none
;
;***************************************;***************************************
;        
AddCharToTXBuffer
		bcf		INTCON,	GIEH			; disable global interrupts
		lfsr	1,		TXBuffer		; use indirect pointer 1 for TXBuffer
		incf	TXBuffer_Head,	W		; next potential empty location in TXBuffer
		movwf	Temp1					; save new head pointer it in Temp1
		movlw	low TXBuffer			; first position in TXBuffer
		addlw	TXBuffer_Size			; add size of buffer
		cpfseq	Temp1					; skip next if pointer overflow
		goto	AddCharToTXBuffer_PointerOK	;	else we are OK, go on
;
AddCharToTXBuffer_PointerOVF
		movlw	low TXBuffer			; first position in TXBuffer 
		movwf	Temp1					; save new head pointer it in Temp1			
;
AddCharToTXBuffer_PointerOK	
		movf	Temp1,	W				; get new head pointer	
		cpfseq	TXBuffer_Tail			; skip next if no more room
		goto	AddCharToTXBuffer_RCBufferRoom	; else we got room, go on
;
AddCharToTXBuffer_RCBufferFull
		bsf		INTCON,	GIEH			; enable global interrupts
		bsf		PIE1,	TX1IE			; enable transmit interrupts
		nop								; a bit of time to allow for the
		nop								; transmit interrupt to occur
		nop								; to make space in the buffer
		goto	AddCharToTXBuffer		; try agian
;
AddCharToTXBuffer_RCBufferRoom		
		movff	TXBuffer_Head,	FSR1L	; point to the empty location
		movff	TX1CharIn,	INDF1		; put the character in
		movff	Temp1,	TXBuffer_Head	; save the new pointer
;
AddCharToTXBuffer_Exit
		bsf		INTCON,	GIEH			; enable global interrupts
		bsf		PIE1,	TX1IE			; enable transmit interrupts	
		return		       
;
;***************************************;***************************************
;
;	Messages
;
;***************************************;***************************************
;
SignOn1
		data	"Jeff Bachiochi",0
;
SignOn2
		data	"FTB286 RFID",0
;
SignOn3
		data	"www.imaginethatnow.com",0
;
Searching
		data	"Searching...",0
;
Finished
		data	"... Done",0
;
NoID
		data	"No ID Found",0
;
Matrix
		data	"Here is the Matrix",0
;
Parity
		data	"Parity Error in Matrix",0	
;
Match
		data	"We've got a match",0	
;
NoMatch
		data	"Sorry, no match",0
;
Unlock
		data	"Door is unlocked",0
;
Lock
		data	"Door is Secured",0	
;
Add
		data	"Adding Codes...",0
;
Saved
		data	"Code Saved",0	
;
AlreadySaved
		data	"Already Saved",0			
;
Delete
		data	"Deleting Codes...",0
;
Deleted
		data	"Code Deleted",0
;
AlreadyGone
		data	"Code Not Here",0																							
;
;***************************************;***************************************
;
;	End
;
;***************************************;***************************************
;
        END